function [pf,pDf] = paramperformindx(tfull,xm,um,pm,optim_param)
% PARAMPERFORMINDX - returns the value of user given objective function for
% tracking problems and its gradients with respect to all optimisation
% variables (li,uij,xij,par) in given collocation point.
% This function is used in cmobjfun, cmobjfungrad functions.

% additional variables
nuj = optim_param.nu*optim_param.ncolu;
nxj = optim_param.nx*(optim_param.ncolx+1);

tmes = optim_param.objtype.tm; xmes = optim_param.objtype.xm;


hdfx = zeros(1,(optim_param.ncolx+1)*optim_param.nx);
hdfu = zeros(1,(optim_param.ncolu)*optim_param.nu);

pf = 0;
if nargout == 2
    pDf = [];
end

% initialisation of variables for calculus of gradients 
if nargout == 2 
    f_row = 1;
    Dft_temp = zeros(f_row,optim_param.dt_col);
    Dfu_temp = zeros(f_row,optim_param.du_col);
    Dfx_temp = zeros(f_row,optim_param.dx_col);
    Dfp_temp = zeros(f_row,optim_param.dp_col);
end


    % actual element value and corresponding measured states
    t_m = tmes';
    x_m = xmes';
    [x_c,u_c,p_c,tau,lfx,lfu,dlfx,srs] = sevalcollpoints(xm,um,pm,optim_param);
    

for i=1:1:length(tmes)
    
    if nargout == 1
        f = feval(optim_param.objfun,t_m(i),x_c(i,:),u_c(i,:),p_c,x_m(i,:));
    else
        [f,Df] = feval(optim_param.objfun,t_m(i),x_c(i,:),u_c(i),p_c,x_m(i,:));
    end
    
    pf = pf + f;
    
    if nargout == 2
      for j=1:1:length(srs)
        if j == 1
            t1 = 0;
            t2 = srs(j);
        else
            t1 = srs(j-1);
            t2 = srs(j);            
        end
        if ((i > t1)&&(i <= t2))
            interval = j;
            if i == t2
                modulus = 0;
            else
                modulus = 1;
            end
        end
    end
    
    
    % gradient of time with respect to the lengths of intervals
            dtdli = dercoltime(optim_param.tau,optim_param.ni,interval);

        
         % gradients of user defined objective function with respect to the
        % element lengths, xij, uij, p
        if ~isempty(Df.t)
            Dft_temp(:,:) = Df.t'*dtdli(:,i)';
        end
        
        if ~isempty(Df.u)
            % nu-by-nu*ncolu matrix
            if modulus == 0
                duduij = kron(eye(optim_param.nu),lfu(i,:));
                in = interval;
                Dfu_temp(:,(in-1)*nuj+1:in*nuj) = Df.u'*duduij + hdfu;
                hdfu = zeros(1,(optim_param.ncolu)*optim_param.nu);
            else
                duduij = kron(eye(optim_param.nu),lfu(i,:));
                in = interval;
                hdfu = hdfu + Df.u'*duduij;
            end            
        end
        
        if ~isempty(Df.x)
            % nx-by-nx*(ncolx+1) matrix
            if modulus == 0
                dxdxij = kron(eye(optim_param.nx),lfx(i,:));
                in = interval;
                Dfx_temp(:,(in-1)*nxj+1:in*nxj) = Df.x'*dxdxij + hdfx;
                hdfx = zeros(1,(optim_param.ncolx+1)*optim_param.nx);
            else
                dxdxij = kron(eye(optim_param.nx),lfx(i,:));
                hdfx = hdfx + Df.x'*dxdxij;                
            end            
        end
        
        if ~isempty(Df.p)
            Dfp_temp(:,:) = Df.p';
        end

        % user defined gradients of objective function
        pDf = [Dft_temp Dfu_temp Dfx_temp Dfp_temp];
           
      end

end
%--------------------------------------------------------------------------